@@ -1,17 +1,36 @@
 #!/usr/bin/env python
+
+# Copyright (C) 2004 Red Hat Inc. <http://www.redhat.com/>
+# Copyright (C) 2005, 2006 Collabora Ltd. <http://www.collabora.co.uk/>
+#
+# Licensed under the Academic Free License version 2.1
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
 import sys
 import os
+import logging
 
-builddir = os.environ["DBUS_TOP_BUILDDIR"]
-pydir = builddir
-
-sys.path.insert(0, pydir)
-sys.path.insert(0, pydir + 'dbus')
+builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
+pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
 
 import dbus
 
 if not dbus.__file__.startswith(pydir):
-    os.system("echo %s> /tmp/dbus.log"%pydir)
     raise Exception("DBus modules are not being picked up from the package")
 
 import dbus.service
@@ -19,6 +38,12 @@ import dbus.glib
 import gobject
 import random
 
+
+logging.basicConfig(filename=builddir + '/test/test-service.log', filemode='w')
+logging.getLogger().setLevel(1)
+logger = logging.getLogger('test-service')
+
+
 class TestInterface(dbus.service.Interface):
     @dbus.service.method("org.freedesktop.DBus.TestSuiteInterface", in_signature='', out_signature='b')
     def CheckInheritance(self):
@@ -34,6 +59,26 @@ class TestObject(dbus.service.Object, TestInterface):
     def Echo(self, arg):
         return arg
 
+    @dbus.service.method("org.freedesktop.DBus.TestSuiteInterface", in_signature='s', out_signature='s')
+    def AcceptUnicodeString(self, foo):
+        assert isinstance(foo, unicode), (foo, foo.__class__.__mro__)
+        return foo
+
+    @dbus.service.method("org.freedesktop.DBus.TestSuiteInterface", in_signature='s', out_signature='s', utf8_strings=True)
+    def AcceptUTF8String(self, foo):
+        assert isinstance(foo, str), (foo, foo.__class__.__mro__)
+        return foo
+
+    @dbus.service.method("org.freedesktop.DBus.TestSuiteInterface", in_signature='ay', out_signature='ay')
+    def AcceptListOfByte(self, foo):
+        assert isinstance(foo, list), (foo, foo.__class__.__mro__)
+        return foo
+
+    @dbus.service.method("org.freedesktop.DBus.TestSuiteInterface", in_signature='ay', out_signature='ay', byte_arrays=True)
+    def AcceptByteArray(self, foo):
+        assert isinstance(foo, str), (foo, foo.__class__.__mro__)
+        return foo
+
     @dbus.service.method("org.freedesktop.DBus.TestSuiteInterface")
     def GetComplexArray(self):
         ret = []
@@ -68,6 +113,7 @@ class TestObject(dbus.service.Object, TestInterface):
 
     @dbus.service.method("org.freedesktop.DBus.TestSuiteInterface", in_signature='u', out_signature='(ss)')
     def ReturnStruct(self, test):
+        logger.info('ReturnStruct(%r) -> %r', test, self.returnValue(test))
         return self.returnValue(test)
 
     @dbus.service.method("org.freedesktop.DBus.TestSuiteInterface", in_signature='u', out_signature='as')
@@ -80,15 +126,15 @@ class TestObject(dbus.service.Object, TestInterface):
 
     @dbus.service.signal("org.freedesktop.DBus.TestSuiteInterface", signature='s')
     def SignalOneString(self, test):
-        pass
+        logger.info('SignalOneString(%r)', test)
 
     @dbus.service.signal("org.freedesktop.DBus.TestSuiteInterface", signature='ss')
     def SignalTwoStrings(self, test, test2):
-        pass
+        logger.info('SignalTwoStrings(%r, %r)', test, test2)
 
     @dbus.service.signal("org.freedesktop.DBus.TestSuiteInterface", signature='(ss)')
     def SignalStruct(self, test):
-        pass
+        logger.info('SignalStruct(%r)', test)
 
     @dbus.service.signal("org.freedesktop.DBus.TestSuiteInterface", signature='as')
     def SignalArray(self, test):
@@ -100,7 +146,7 @@ class TestObject(dbus.service.Object, TestInterface):
 
     @dbus.service.method("org.freedesktop.DBus.TestSuiteInterface", in_signature='su', out_signature='')
     def EmitSignal(self, signal, value):
-        sig = getattr(self, signal, None)
+        sig = getattr(self, str(signal), None)
         assert(sig != None)
 
         val = self.returnValue(value)
@@ -110,6 +156,7 @@ class TestObject(dbus.service.Object, TestInterface):
         else:
             val = tuple([val])
 
+        logger.info('Emitting %s with %r', signal, val)
         sig(*val)
 
     def CheckInheritance(self):
